//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/MaskEditorCanvas.cpp
//! @brief     Implements class MaskEditorCanvas.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Canvas/MaskEditorCanvas.h"
#include "Device/Data/Datafield.h"
#include "GUI/Model/Data/Data2DItem.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/MaskList.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/PlotScale/SavePlotAssistant.h"
#include "GUI/View/Plotter/ColorMap.h"
#include "GUI/View/Plotter/PlotStatusLabel.h"
#include "GUI/View/Scene/MaskGraphicsScene.h"
#include "GUI/View/Scene/MaskGraphicsView.h"
#include <QVBoxLayout>

MaskEditorCanvas::MaskEditorCanvas()
    : m_scene(new MaskGraphicsScene)
    , m_view(new MaskGraphicsView(m_scene))
    , m_statusLabel(new PlotStatusLabel(nullptr, this))
    , m_data2DItem(nullptr)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    auto* layout = new QVBoxLayout;
    setLayout(layout);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    layout->addWidget(m_view);
    layout->addWidget(m_statusLabel);

    connect(m_view, &MaskGraphicsView::changeActivityRequest, this,
            &MaskEditorCanvas::changeActivityRequest);
    connect(m_view, &MaskGraphicsView::deleteSelectedRequest, this,
            &MaskEditorCanvas::deleteSelectedRequest);
}

MaskEditorCanvas::~MaskEditorCanvas() = default;

void MaskEditorCanvas::updateMaskCanvas(Data2DItem* data2DItem, QItemSelectionModel* selModel)
{
    m_data2DItem = data2DItem;

    m_scene->associateItems(data2DItem, data2DItem->getOrCreateMaskModel(), selModel);

    repaintScene();

    m_statusLabel->addPlot(m_scene->colorMap());
}

void MaskEditorCanvas::resetMaskCanvas()
{
    m_data2DItem = nullptr;
    m_scene->dissociateItems();
    m_statusLabel->reset();
}

void MaskEditorCanvas::onPresentationChange(bool pixelized)
{
    m_scene->clearSelection(); // important to avoid crash (unsubscribe while calling subscribers)

    ASSERT(m_data2DItem);

    if (pixelized) {
        if (Datafield* maskedData = m_data2DItem->createMaskedField()) {
            // store data backup
            m_backup_data.reset(m_data2DItem->c_field()->clone());
            m_backup_interpolated = m_data2DItem->isInterpolated();

            m_data2DItem->setDatafield(*maskedData);
            m_data2DItem->setInterpolated(false);
        } else {
            m_backup_data.reset();
        }

    } else {
        if (m_backup_data) {
            m_data2DItem->setDatafield(*m_backup_data);
            m_data2DItem->setInterpolated(m_backup_interpolated);
        }
    }
    repaintScene();

    if (auto* container = m_data2DItem->maskContainerItem())
        for (MaskItem* mask : container->modifiableMaskItems())
            mask->setIsVisible(!pixelized && mask->wasVisible());
}

void MaskEditorCanvas::repaintScene()
{
    m_scene->updateSize(m_view->size()); // TODO replace by proper repaint command
}

//! Saves plot into project directory.

void MaskEditorCanvas::onSavePlotRequest()
{
    ASSERT(gDoc);
    QString dirname = gDoc->userExportDir();
    GUI::Plot::savePlot(dirname, m_scene->colorMap()->customPlot(), m_data2DItem->c_field());
}

void MaskEditorCanvas::onResetViewRequest()
{
    m_view->onResetViewRequest();

    if (m_data2DItem->isZoomed())
        m_data2DItem->resetView();
    else
        setZoomToROI();
    gDoc->setModified();
}

void MaskEditorCanvas::setZoomToROI()
{
    if (MaskList* item = m_data2DItem->maskContainerItem()) {
        if (auto* roiItem = item->regionOfInterestItem()) {
            m_data2DItem->setXrange(roiItem->xLow(), roiItem->xUp());
            m_data2DItem->setYrange(roiItem->yLow(), roiItem->yUp());
        }
    }
}
